"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpTransport = void 0;
const node_url_1 = require("node:url");
const http = require("node:http");
const https = require("node:https");
const Transport = require("winston-transport");
function getServer(url) {
    const info = new node_url_1.URL(url);
    return {
        ssl: info.protocol === 'https:',
        host: info.hostname,
        port: info.port ? Number(info.port) : undefined,
        path: info.pathname,
    };
}
class HttpTransport extends Transport {
    constructor(options) {
        super(options);
        this.options = options;
    }
    log(info, next) {
        this.request(info, (err, res) => {
            if (res && res.statusCode !== 200) {
                err = new Error(`Invalid HTTP Status Code: ${res.statusCode}`);
            }
            if (err) {
                this.emit('warn', err);
            }
            else {
                this.emit('logged', info);
            }
        });
        if (next) {
            setImmediate(next);
        }
    }
    request(info, callback) {
        const server = getServer(this.options.url);
        const headers = Object.assign({
            'Content-Type': 'application/json; charset=utf-8',
        }, this.options.headers);
        const auth = this.options.auth;
        if (auth && auth.bearer) {
            headers.Authorization = `Bearer ${auth.bearer}`;
        }
        const req = (server.ssl ? https : http).request({
            method: 'POST',
            host: server.host,
            port: server.port,
            path: server.path,
            headers,
            auth: auth && auth.username && auth.password
                ? `${auth.username}:${auth.password}`
                : '',
        });
        req.on('error', callback);
        req.on('response', (res) => res.on('end', () => callback(null, res)).resume());
        req.end(Buffer.from(info[Symbol.for('message')], 'utf8'));
    }
}
exports.HttpTransport = HttpTransport;
